@@ -51,9 +51,10 @@ namespace Pinta.Core
 		private Layer selection_layer;
 
 		private int selection_layer_index;
-		private Path selection_path;
 		private bool show_selection;
 
+		public DocumentSelection Selection = new DocumentSelection();
+
 		public Document (Gdk.Size size)
 		{
 			Guid = Guid.NewGuid ();
@@ -140,16 +141,6 @@ namespace Pinta.Core
 		public Layer SelectionLayer {
 			get { return selection_layer; }
 		}
-		
-		public Path SelectionPath {
-			get { return selection_path; }
-			set {
-				if (selection_path == value)
-					return;
-
-				selection_path = value;
-			}
-		}
 
 		public bool ShowSelection {
 			get { return show_selection; }
@@ -241,8 +232,7 @@ namespace Pinta.Core
 			if (selection_layer != null)
 				(selection_layer.Surface as IDisposable).Dispose ();
 
-			if (selection_path != null)
-				(selection_path as IDisposable).Dispose ();
+			Selection.DisposeSelection();
 
             Workspace.History.Clear ();
 		}
@@ -251,7 +241,7 @@ namespace Pinta.Core
 		{
 			Context g = new Context (CurrentLayer.Surface);
 
-			g.AppendPath (SelectionPath);
+			g.AppendPath (Selection.SelectionPath);
 			g.FillRule = Cairo.FillRule.EvenOdd;
 			g.Clip ();
 
@@ -262,7 +252,7 @@ namespace Pinta.Core
 		{
 			Context g = new Context (CurrentLayer.Surface);
 
-			g.AppendPath (SelectionPath);
+			g.AppendPath(Selection.SelectionPath);
 			g.FillRule = Cairo.FillRule.EvenOdd;
 			g.Clip ();
 
@@ -275,7 +265,7 @@ namespace Pinta.Core
 		{
 			Context g = new Context (ToolLayer.Surface);
 
-			g.AppendPath (SelectionPath);
+			g.AppendPath(Selection.SelectionPath);
 			g.FillRule = Cairo.FillRule.EvenOdd;
 			g.Clip ();
 
@@ -286,7 +276,7 @@ namespace Pinta.Core
 		{
 			Context g = new Context (ToolLayer.Surface);
 
-			g.AppendPath (SelectionPath);
+			g.AppendPath(Selection.SelectionPath);
 			g.FillRule = Cairo.FillRule.EvenOdd;
 			g.Clip ();
 
@@ -422,23 +412,21 @@ namespace Pinta.Core
 			if (Layers.Count < 2)
 				throw new InvalidOperationException ("Cannot flatten image because there is only one layer.");
 
-			Layer dest = Layers[0];
+			// Find the "bottom" layer
+			var bottom_layer = Layers[0];
+			var old_surf = bottom_layer.Surface;
 
-			using (Cairo.Context g = new Cairo.Context (dest.Surface)) {
-				for (int i = 1; i < Layers.Count; i++) {
-					Layer source = Layers[i];
-					g.SetSource (source.Surface);
-					g.PaintWithAlpha (source.Opacity);
-				}
-			}
+			// Replace the bottom surface with the flattened image,
+			// and dispose the old surface
+			bottom_layer.Surface = GetFlattenedImage ();
+			(old_surf as IDisposable).Dispose ();
 
+			// Reset our layer pointer to the only remaining layer
 			current_layer = 0;
 
-			while (Layers.Count > 1) {
-				Layer l = Layers[1];
-
+			// Delete all other layers
+			while (Layers.Count > 1)
 				Layers.RemoveAt (1);
-			}
 
 			PintaCore.Layers.OnLayerRemoved ();
 			Workspace.Invalidate ();
@@ -467,7 +455,7 @@ namespace Pinta.Core
 			Cairo.ImageSurface surf = new Cairo.ImageSurface (Cairo.Format.Argb32, ImageSize.Width, ImageSize.Height);
 
 			using (Cairo.Context g = new Cairo.Context (surf)) {
-				g.AppendPath (SelectionPath);
+				g.AppendPath(Selection.SelectionPath);
 				g.Clip ();
 
 				g.SetSource (Layers[index].Surface);
@@ -477,17 +465,34 @@ namespace Pinta.Core
 			return surf;
 		}
 
-		public ImageSurface GetFlattenedImage ()
+		/// <summary>
+		/// Gets the final pixel color for the given point, taking layers, opacity, and blend modes into account.
+		/// </summary>
+		public ColorBgra GetComputedPixel (int x, int y)
 		{
-			Cairo.ImageSurface surf = new Cairo.ImageSurface (Cairo.Format.Argb32, ImageSize.Width, ImageSize.Height);
+			var pixel = ColorBgra.Zero;
 
-			using (Cairo.Context g = new Cairo.Context (surf)) {
-				foreach (var layer in GetLayersToPaint ()) {
-					g.SetSource (layer.Surface);
-					g.PaintWithAlpha (layer.Opacity);
-				}
+			foreach (var layer in GetLayersToPaint ()) {
+				var blend_op = UserBlendOps.GetBlendOp (layer.BlendMode, layer.Opacity);
+
+				pixel = blend_op.Apply (pixel, layer.Surface.GetColorBgra (x, y));
 			}
 
+			return pixel;
+		}
+
+		public ImageSurface GetFlattenedImage ()
+		{
+			// Create a new image surface
+			var surf = new Cairo.ImageSurface (Cairo.Format.Argb32, ImageSize.Width, ImageSize.Height);
+
+			// Blend each visible layer onto our surface
+			foreach (var layer in GetLayersToPaint ()) {
+				var blendop = UserBlendOps.GetBlendOp (layer.BlendMode, layer.Opacity);
+				blendop.Apply (surf, layer.Surface);
+			}
+
+			surf.MarkDirty ();
 			return surf;
 		}
 
@@ -514,7 +519,7 @@ namespace Pinta.Core
 		/// <param name="canvasOnly">false for the whole selection, true for the part only on our canvas</param>
 		public Gdk.Rectangle GetSelectedBounds (bool canvasOnly)
 		{
-			var bounds = SelectionPath.GetBounds ();
+			var bounds = Selection.SelectionPath.GetBounds();
 
 			if (canvasOnly)
 				bounds = ClampToImageSize (bounds);
@@ -546,13 +551,13 @@ namespace Pinta.Core
 			if (current_layer == 0)
 				throw new InvalidOperationException ("Cannot flatten layer because current layer is the bottom layer.");
 
-			Layer source = CurrentLayer;
-			Layer dest = Layers[current_layer - 1];
+			// Get our source and destination layers
+			var source = CurrentLayer;
+			var dest = Layers[current_layer - 1];
 
-			using (Cairo.Context g = new Cairo.Context (dest.Surface)) {
-				g.SetSource (source.Surface);
-				g.PaintWithAlpha (source.Opacity);
-			}
+			// Blend the layers
+			var blendop = UserBlendOps.GetBlendOp (source.BlendMode, source.Opacity);
+			blendop.Apply (dest.Surface, source.Surface);
 
 			DeleteCurrentLayer ();
 		}
@@ -587,20 +592,23 @@ namespace Pinta.Core
 			Workspace.Invalidate ();
 		}
 		
-		public void ResetSelectionPath ()
+		public void ResetSelectionPath()
 		{
-			Path old = SelectionPath;
+			Selection.DisposeSelectionPreserve();
 
-			using (Cairo.Context g = new Cairo.Context (selection_layer.Surface))
-				SelectionPath = g.CreateRectanglePath (new Cairo.Rectangle (0, 0, ImageSize.Width, ImageSize.Height));
-
-			if (old != null)
-				(old as IDisposable).Dispose ();
+			Selection.ResetSelection(selection_layer.Surface, ImageSize);
 
 			ShowSelection = false;
 		}
 
-		public void ResizeCanvas (int width, int height, Anchor anchor)
+		/// <summary>
+		/// Resizes the canvas.
+		/// </summary>
+		/// <param name='compoundAction'>
+		/// Optionally, the history item for resizing the canvas can be added to
+		/// a CompoundHistoryItem if it is part of a larger action (e.g. pasting an image).
+		/// </param>
+		public void ResizeCanvas (int width, int height, Anchor anchor, CompoundHistoryItem compoundAction)
 		{
 			double scale;
 
@@ -612,7 +620,7 @@ namespace Pinta.Core
 			ResizeHistoryItem hist = new ResizeHistoryItem (ImageSize);
 			hist.Icon = "Menu.Image.CanvasSize.png";
 			hist.Text = Catalog.GetString ("Resize Canvas");
-			hist.TakeSnapshotOfImage ();
+			hist.StartSnapshotOfImage ();
 
 			scale = Workspace.Scale;
 
@@ -621,7 +629,13 @@ namespace Pinta.Core
 			foreach (var layer in Layers)
 				layer.ResizeCanvas (width, height, anchor);
 
-			Workspace.History.PushNewItem (hist);
+			hist.FinishSnapshotOfImage ();
+
+			if (compoundAction != null) {
+				compoundAction.Push (hist);
+			} else {
+				Workspace.History.PushNewItem (hist);
+			}
 
 			ResetSelectionPath ();
 
@@ -638,7 +652,7 @@ namespace Pinta.Core
 			PintaCore.Tools.Commit ();
 
 			ResizeHistoryItem hist = new ResizeHistoryItem (ImageSize);
-			hist.TakeSnapshotOfImage ();
+			hist.StartSnapshotOfImage ();
 
 			scale = Workspace.Scale;
 
@@ -647,6 +661,8 @@ namespace Pinta.Core
 			foreach (var layer in Layers)
 				layer.Resize (width, height);
 
+			hist.FinishSnapshotOfImage ();
+
 			Workspace.History.PushNewItem (hist);
 
 			ResetSelectionPath ();
@@ -695,16 +711,17 @@ namespace Pinta.Core
 
 		public void SetCurrentLayer (int i)
 		{
-			current_layer = i;
+			// Ensure that the current tool's modifications are finalized before
+			// switching layers.
+			PintaCore.Tools.CurrentTool.DoCommit ();
 
+			current_layer = i;
 			PintaCore.Layers.OnSelectedLayerChanged ();
 		}
 
 		public void SetCurrentLayer (Layer layer)
 		{
-			current_layer = Layers.IndexOf (layer);
-
-			PintaCore.Layers.OnSelectedLayerChanged ();
+			SetCurrentLayer (Layers.IndexOf (layer));
 		}
 		#endregion
 
