@@ -854,7 +854,7 @@ namespace Pinta.Core
 			Rectangle rect;
 
 			using (Context g = new Context (PintaCore.Layers.CurrentLayer.Surface)) {
-				g.AppendPath (PintaCore.Layers.SelectionPath);
+				g.AppendPath (PintaCore.Workspace.ActiveDocument.Selection.SelectionPath);
 
 				// We don't want the bounding box to include a stroke width 
 				// of 1, but setting it to 0 returns an empty rectangle.  Set
@@ -997,7 +997,7 @@ namespace Pinta.Core
 		/// </summary>
 		public static bool ContainsCorrect (this Gdk.Rectangle r, int x, int y)
 		{
-			return ((((x >= r.Left) && (x < r.Right)) && (y >= r.Top)) && (y < r.Bottom));
+			return ((((x >= r.Left) && (x <= r.GetRight ())) && (y >= r.Top)) && (y <= r.GetBottom ()));
 		}
 
 		/// <summary>
@@ -1406,5 +1406,10 @@ namespace Pinta.Core
 		{
 			return new Gdk.Point ((int)point.X, (int)point.Y);
 		}
+
+		public static bool IsEmpty (this PointD point)
+		{
+			return point.X == 0 && point.Y == 0;
+		}
 	}
 }
