@@ -42,8 +42,8 @@ namespace Pinta.Core
 			Icon = "Menu.Image.Resize.png";
 			Text = Catalog.GetString ("Resize Image");
 		}
-		
-		public Cairo.Path RestorePath { get; set; }
+
+		public DocumentSelection RestoreSelection;
 		
 		public override void Undo ()
 		{
@@ -59,14 +59,14 @@ namespace Pinta.Core
 			
 			base.Undo ();
 			
-			if (RestorePath != null) {
-				Cairo.Path old = PintaCore.Layers.SelectionPath;
+			if (RestoreSelection != null) {
+				DocumentSelection old = PintaCore.Workspace.ActiveDocument.Selection;
+				PintaCore.Workspace.ActiveDocument.Selection = RestoreSelection.Clone();
+
+				if (old != null) {
+					old.DisposeSelection ();
+				}
 
-				PintaCore.Layers.SelectionPath = RestorePath.Clone ();
-				
-				if (old != null)
-					(old as IDisposable).Dispose ();
-					
 				PintaCore.Layers.ShowSelection = true;
 			} else {
 				PintaCore.Layers.ResetSelectionPath ();
@@ -101,9 +101,8 @@ namespace Pinta.Core
 		{
 			base.Dispose ();
 
-			if (RestorePath != null) {
-				(RestorePath as IDisposable).Dispose ();
-				RestorePath = null;
+			if (RestoreSelection != null) {
+				RestoreSelection.DisposeSelection ();
 			}
 		}
 	}
