@@ -85,7 +85,7 @@ namespace Pinta.Core
 			
 			// Handle selection path.
 			PintaCore.Tools.Commit ();
-			selection_path = (PintaCore.Layers.ShowSelection) ? PintaCore.Layers.SelectionPath : null;
+			selection_path = (PintaCore.Layers.ShowSelection) ? PintaCore.Workspace.ActiveDocument.Selection.SelectionPath : null;
 			render_bounds = selection_path.GetBounds ();
 			render_bounds = PintaCore.Workspace.ClampToImageSize (render_bounds);			
 									
@@ -241,7 +241,7 @@ namespace Pinta.Core
 			using (var ctx = new Cairo.Context (layer.Surface)) {
 				
 				ctx.Save ();
-				ctx.AppendPath (PintaCore.Layers.SelectionPath);
+				ctx.AppendPath (PintaCore.Workspace.ActiveDocument.Selection.SelectionPath);
 				ctx.FillRule = Cairo.FillRule.EvenOdd;
 				ctx.Clip ();				
 			
@@ -355,8 +355,8 @@ namespace Pinta.Core
 			// Calculate canvas bounds.
 			double x1 = bounds.Left * scale;
 			double y1 = bounds.Top * scale;
-			double x2 = bounds.Right * scale;
-			double y2 = bounds.Bottom * scale;
+			double x2 = (bounds.GetRight () + 1) * scale;
+			double y2 = (bounds.GetBottom () + 1) * scale;
 
 			// TODO Figure out why when scale > 1 that I need add on an
 			// extra pixel of padding.
