@@ -29,12 +29,16 @@ using Cairo;
 using Gtk;
 using Pinta.Core;
 using Mono.Unix;
+using ClipperLibrary;
+using System.Collections.Generic;
+using System.Linq;
 
 namespace Pinta.Tools
 {
 	public class LassoSelectTool : SelectTool
 	{
 		private Path path;
+		private List<IntPoint> lassoPolygon = new List<IntPoint>();
 
 		public LassoSelectTool ()
 		{
@@ -80,15 +84,16 @@ namespace Pinta.Tools
 					
 				g.LineTo (x, y);
 
+				lassoPolygon.Add(new IntPoint((long)x, (long)y));
+
 				path = g.CopyPath ();
 				
 				g.FillRule = FillRule.EvenOdd;
 				g.ClosePath ();
 
-				Path old = doc.SelectionPath;
+				doc.Selection.DisposeSelectionPreserve();
 
-				doc.SelectionPath = g.CopyPath (); 
-				(old as IDisposable).Dispose ();
+				doc.Selection.SelectionPath = g.CopyPath ();
 			}
 
 			doc.Workspace.Invalidate ();
@@ -112,12 +117,14 @@ namespace Pinta.Tools
 				g.FillRule = FillRule.EvenOdd;
 				g.ClosePath ();
 
-				Path old = doc.SelectionPath;
+				doc.Selection.DisposeSelectionPreserve();
 
-				doc.SelectionPath = g.CopyPath ();
-				(old as IDisposable).Dispose ();
+				doc.Selection.SelectionPath = g.CopyPath ();
 			}
 
+			doc.Selection.SelectionPolygons.Add(lassoPolygon.ToList());
+			lassoPolygon.Clear();
+
 			doc.Workspace.Invalidate ();
 		}
 		#endregion
