@@ -120,7 +120,7 @@ namespace Pinta.Tools
 				surf.MarkDirty ();
 			} else {
 				using (Context g = new Context (surf)) {
-					g.AppendPath (doc.SelectionPath);
+					g.AppendPath (doc.Selection.SelectionPath);
 					g.FillRule = FillRule.EvenOdd;
 					g.Clip ();
 				
@@ -141,7 +141,11 @@ namespace Pinta.Tools
 			
 			Gdk.Rectangle r = GetRectangleFromPoints (last_point, new Point (x, y));
 
-			doc.Workspace.Invalidate (r);
+			if (doc.Workspace.IsPartiallyOffscreen (r)) {
+				doc.Workspace.Invalidate ();
+			} else {
+				doc.Workspace.Invalidate (r);
+			}
 			
 			last_point = new Point (x, y);
 		}
