@@ -51,15 +51,15 @@ namespace Pinta.Tools
 			engine = new TextEngine ();
 		}
 
-        static TextTool ()
-        {
-            Gtk.IconFactory fact = new Gtk.IconFactory ();
-            fact.Add ("ShapeTool.Outline.png", new Gtk.IconSet (PintaCore.Resources.GetIcon ("ShapeTool.Outline.png")));
-            fact.Add ("ShapeTool.Fill.png", new Gtk.IconSet (PintaCore.Resources.GetIcon ("ShapeTool.Fill.png")));
-            fact.Add ("ShapeTool.OutlineFill.png", new Gtk.IconSet (PintaCore.Resources.GetIcon ("ShapeTool.OutlineFill.png")));
-            fact.Add ("TextTool.FillBackground.png", new Gtk.IconSet (PintaCore.Resources.GetIcon ("TextTool.FillBackground.png")));
-            fact.AddDefault ();
-        }
+		static TextTool ()
+		{
+			Gtk.IconFactory fact = new Gtk.IconFactory ();
+			fact.Add ("ShapeTool.Outline.png", new Gtk.IconSet (PintaCore.Resources.GetIcon ("ShapeTool.Outline.png")));
+			fact.Add ("ShapeTool.Fill.png", new Gtk.IconSet (PintaCore.Resources.GetIcon ("ShapeTool.Fill.png")));
+			fact.Add ("ShapeTool.OutlineFill.png", new Gtk.IconSet (PintaCore.Resources.GetIcon ("ShapeTool.OutlineFill.png")));
+			fact.Add ("TextTool.FillBackground.png", new Gtk.IconSet (PintaCore.Resources.GetIcon ("TextTool.FillBackground.png")));
+			fact.AddDefault ();
+		}
 		#endregion
 
 		#region ToolBar
@@ -554,27 +554,27 @@ namespace Pinta.Tools
 						break;
 
 					case Gdk.Key.Left:
-						engine.PerformLeft ((modifier & Gdk.ModifierType.ControlMask) != 0);
+						engine.PerformLeft ((modifier & Gdk.ModifierType.ControlMask) != 0, (modifier & Gdk.ModifierType.ShiftMask) != 0);
 						break;
 
 					case Gdk.Key.Right:
-						engine.PerformRight ((modifier & Gdk.ModifierType.ControlMask) != 0);
+						engine.PerformRight ((modifier & Gdk.ModifierType.ControlMask) != 0, (modifier & Gdk.ModifierType.ShiftMask) != 0);
 						break;
 
 					case Gdk.Key.Up:
-						engine.PerformUp ();
+						engine.PerformUp ((modifier & Gdk.ModifierType.ShiftMask) != 0);
 						break;
 
 					case Gdk.Key.Down:
-						engine.PerformDown ();
+						engine.PerformDown ((modifier & Gdk.ModifierType.ShiftMask) != 0);
 						break;
 
 					case Gdk.Key.Home:
-						engine.PerformHome ((modifier & Gdk.ModifierType.ControlMask) != 0);
+						engine.PerformHome ((modifier & Gdk.ModifierType.ControlMask) != 0, (modifier & Gdk.ModifierType.ShiftMask) != 0);
 						break;
 
 					case Gdk.Key.End:
-						engine.PerformEnd ((modifier & Gdk.ModifierType.ControlMask) != 0);
+						engine.PerformEnd ((modifier & Gdk.ModifierType.ControlMask) != 0, (modifier & Gdk.ModifierType.ShiftMask) != 0);
 						break;
 
 					case Gdk.Key.Next:
@@ -584,19 +584,31 @@ namespace Pinta.Tools
 					case Gdk.Key.Escape:
 						StopEditing ();
 						break;
-
+					case Gdk.Key.Insert:
+						if ((modifier & Gdk.ModifierType.ShiftMask) != 0) {
+							Gtk.Clipboard cb = Gtk.Clipboard.Get (Gdk.Atom.Intern ("CLIPBOARD", false));
+							engine.PerformPaste (cb);
+						} else if ((modifier & Gdk.ModifierType.ControlMask) != 0) {
+							Gtk.Clipboard cb = Gtk.Clipboard.Get (Gdk.Atom.Intern ("CLIPBOARD", false));
+							engine.PerformCopy (cb);
+						}
+						break;
 					default:
-						// Try to handle it as a character
-						uint ch = Gdk.Keyval.ToUnicode (args.Event.KeyValue);
+						// Ignore command shortcut
+						if ((modifier & Gdk.ModifierType.ControlMask) != 0) {
+							// Handle undo while editing - if there is nothing on the history stack but
+							// the user has typed some text, we need to handle the shortcut ourselves.
+							if (args.Event.Key == Gdk.Key.z) {
+								StopEditing ();
 
-						if (ch != 0) {
-							engine.InsertCharIntoString (ch);
-							RedrawText (true, true);
-						} else {
-							// We didn't handle the key
-							keyHandled = false;
+								if (PintaCore.Workspace.ActiveDocument.History.CanUndo) {
+									PintaCore.Workspace.ActiveDocument.History.Undo ();
+								}
+							}
+							return;
 						}
 
+						keyHandled = TryHandleChar(args.Event);
 						break;
 				}
 
@@ -605,14 +617,26 @@ namespace Pinta.Tools
 					RedrawText (true, true);
 
 			}
-            else
-            {
-                // If we're not editing, allow the key press to be handled elsewhere (e.g. for selecting another tool).
-                keyHandled = false;
-            }
+			else
+			{
+				// If we're not editing, allow the key press to be handled elsewhere (e.g. for selecting another tool).
+				keyHandled = false;
+			}
 
 			args.RetVal = keyHandled;
 		}
+
+		private bool TryHandleChar(EventKey eventKey)
+		{
+			// Try to handle it as a character
+			if (engine.HandleKeyPress (eventKey)) {
+				RedrawText (true, true);
+				return true;
+			}
+
+			// We didn't handle the key
+			return false;
+		}
 		#endregion
 
 		#region Start/Stop Editing
@@ -675,7 +699,14 @@ namespace Pinta.Tools
 			using (var g = new Cairo.Context (surf)) {
 				g.Save ();
 
-				g.AppendPath (PintaCore.Workspace.ActiveDocument.SelectionPath);
+				// Show selection if on tool layer
+				if (useToolLayer) {
+					// Selected Text
+					Cairo.Color c = new Cairo.Color (0.7, 0.8, 0.9, 0.5);
+					foreach (Rectangle rect in engine.SelectionRectangles)
+						g.FillRectangle (rect.ToCairoRectangle (), c);
+				}
+				g.AppendPath (PintaCore.Workspace.ActiveDocument.Selection.SelectionPath);
 				g.FillRule = Cairo.FillRule.EvenOdd;
 				g.Clip ();
 
@@ -730,5 +761,50 @@ namespace Pinta.Tools
 			old_bounds = r;
 		}
 		#endregion
+		#region undo
+
+		public override bool TryHandleUndo ()
+		{
+			if (engine.EditMode == EditingMode.NotEditing) {
+				return false;
+			}
+			// commit an history item to let the undo action undo text history item
+			StopEditing ();
+			return false;
+		}
+
+		#endregion
+		#region Copy/Paste
+
+		public override bool TryHandlePaste (Clipboard cb)
+		{
+			if (engine.EditMode == EditingMode.NotEditing) {
+				return false;
+			}
+			engine.PerformPaste (cb);
+			RedrawText (true, true);
+			return true;
+		}
+
+		public override bool TryHandleCopy (Clipboard cb)
+		{
+			if (engine.EditMode == EditingMode.NotEditing) {
+				return false;
+			}
+			engine.PerformCopy (cb);
+			return true;
+		}
+
+		public override bool TryHandleCut (Clipboard cb)
+		{
+			if (engine.EditMode == EditingMode.NotEditing) {
+				return false;
+			}
+			engine.PerformCut (cb);
+			RedrawText (true, true);
+			return true;
+		}
+
+		#endregion#endregion
 	}
 }
