@@ -93,6 +93,7 @@ namespace Pinta
 				PintaCore.Actions.File.Save.Sensitive = false;
 				PintaCore.Actions.File.SaveAs.Sensitive = false;
 				PintaCore.Actions.Edit.Copy.Sensitive = false;
+				PintaCore.Actions.Edit.CopyMerged.Sensitive = false;
 				PintaCore.Actions.Edit.Cut.Sensitive = false;
 				PintaCore.Actions.Edit.Paste.Sensitive = false;
 				PintaCore.Actions.Edit.PasteIntoNewLayer.Sensitive = false;
@@ -125,6 +126,9 @@ namespace Pinta
 
 				PintaCore.Actions.Adjustments.ToggleActionsSensitive (false);
 				PintaCore.Actions.Effects.ToggleActionsSensitive (false);
+
+				PintaCore.Actions.Window.SaveAll.Sensitive = false;
+				PintaCore.Actions.Window.CloseAll.Sensitive = false;
 			}
 		}
 
@@ -136,6 +140,7 @@ namespace Pinta
 			PintaCore.Actions.File.Save.Sensitive = true;
 			PintaCore.Actions.File.SaveAs.Sensitive = true;
 			PintaCore.Actions.Edit.Copy.Sensitive = true;
+			PintaCore.Actions.Edit.CopyMerged.Sensitive = true;
 			PintaCore.Actions.Edit.Cut.Sensitive = true;
 			PintaCore.Actions.Edit.Paste.Sensitive = true;
 			PintaCore.Actions.Edit.PasteIntoNewLayer.Sensitive = true;
@@ -167,6 +172,9 @@ namespace Pinta
 
 			PintaCore.Actions.Adjustments.ToggleActionsSensitive (true);
 			PintaCore.Actions.Effects.ToggleActionsSensitive (true);
+
+			PintaCore.Actions.Window.SaveAll.Sensitive = true;
+			PintaCore.Actions.Window.CloseAll.Sensitive = true;
 		}
 	}
 }
